var DefaultLayout = function(){this.init.apply(this, arguments);};
exports.DefaultLayout = DefaultLayout;

DefaultLayout.prototype.init = function() {};

DefaultLayout.prototype.format = function(_logEvent) {

	return _logEvent.getMessage();
};

DefaultLayout.prototype.getContentType = function() {

	return "text/plain";
};

DefaultLayout.prototype.getHeader = function() {

	return "";
};

DefaultLayout.prototype.getFooter = function() {

	return "";
};
var StringBuilder = require("bn-lang").StringBuilder;

var DefaultAppender = function(){this.init.apply(this, arguments);};
exports.DefaultAppender = DefaultAppender;

DefaultAppender.prototype.init = function(_name, _output) {

	this.name = _name;
	this.output = _output;
	this.layout = new DefaultLayout();
	this.threshold = Level.ALL;
};

DefaultAppender.prototype.doAppend = function(_logEvent) {

	if(_logEvent.getLevel().isGreaterOrEqual(this.getThreshold())) {

		this.append(_logEvent);
	}
};

DefaultAppender.prototype.append = function(_logEvent) {

	var layout = this.getLayout();

	this.output(new StringBuilder()
				.append(layout.getHeader())
				.append(layout.format(_logEvent))
				.append(layout.getFooter())
				.append("\n")
				.toString());
};

DefaultAppender.prototype.getName = function() {

	return this.name;
};

DefaultAppender.prototype.getLayout = function() {

	return this.layout;
};

DefaultAppender.prototype.setLayout = function(_layout) {

	this.layout = _layout;
};

DefaultAppender.prototype.getThreshold = function() {

	return this.threshold;
};

DefaultAppender.prototype.setThreshold = function(_threshold) {

	this.threshold = _threshold;
};
var Level = function(){this.init.apply(this, arguments);};
exports.Level = Level;

Level.prototype.init = function(_number, _name) {

	this.number = _number;
	this.name = _name;
};

Level.prototype.getNumber = function() {

	return this.number;
};

Level.prototype.getName = function() {

	return this.name;
};

Level.prototype.isGreaterOrEqual = function(_level) {

	return this.getNumber() >= _level.getNumber();
};

Level.prototype.toString = function() {

	return this.getName();
};

Level.prototype.equals = function(_object) {

	if (this === _object) {

		return true;
	}
	if(this.constructor != _object.constructor) {

		return false;
	}
	return this.getNumber() == _object.getNumber();
};

Level.toLevel = function(_value) {

	if(_value.constructor == String) {

		if(Level.ALL_VALUE.equals(_value)){return Level.ALL;}
		else if(Level.TRACE_VALUE.equals(_value)){return Level.TRACE;}
		else if(Level.DEBUG_VALUE.equals(_value)){return Level.DEBUG;}
		else if(Level.INFO_VALUE.equals(_value)){return Level.INFO;}
		else if(Level.WARN_VALUE.equals(_value)){return Level.WARN;}
		else if(Level.ERROR_VALUE.equals(_value)){return Level.ERROR;}
		else if(Level.FATAL_VALUE.equals(_value)){return Level.FATAL;}
		else if(Level.OFF_VALUE.equals(_value)){return Level.OFF;}
		else {return null;}

	} else if(_value.constructor == Number) {

		switch(_value) {

			case Level.ALL_INT:
				return Level.ALL;
			case Level.TRACE_INT:
				return Level.TRACE;
			case Level.DEBUG_INT:
				return Level.DEBUG;
			case Level.INFO_INT:
				return Level.INFO;
			case Level.WARN_INT:
				return Level.WARN;
			case Level.ERROR_INT:
				return Level.ERROR;
			case Level.FATAL_INT:
				return Level.FATAL;
			case Level.OFF_INT:
				return Level.OFF;
			default:
				return null;
		}

	} else {

		return null;
	}
};

Level.ALL_INT	= Number.MIN_VALUE;
Level.TRACE_INT	= 100;
Level.DEBUG_INT	= 200;
Level.INFO_INT	= 300;
Level.WARN_INT	= 400;
Level.ERROR_INT	= 500;
Level.FATAL_INT	= 600;
Level.OFF_INT	= Number.MAX_VALUE;

Level.ALL_VALUE		= "ALL";
Level.TRACE_VALUE	= "TRACE";
Level.DEBUG_VALUE	= "DEBUG";
Level.INFO_VALUE	= "INFO";
Level.WARN_VALUE	= "WARN";
Level.ERROR_VALUE	= "ERROR";
Level.FATAL_VALUE	= "FATAL";
Level.OFF_VALUE		= "OFF";

Level.ALL	= new Level(Level.ALL_INT, Level.ALL_VALUE);
Level.TRACE	= new Level(Level.TRACE_INT, Level.TRACE_VALUE);
Level.DEBUG	= new Level(Level.DEBUG_INT, Level.DEBUG_VALUE);
Level.INFO	= new Level(Level.INFO_INT, Level.INFO_VALUE);
Level.WARN	= new Level(Level.WARN_INT, Level.WARN_VALUE);
Level.ERROR	= new Level(Level.ERROR_INT, Level.ERROR_VALUE);
Level.FATAL	= new Level(Level.FATAL_INT, Level.FATAL_VALUE);
Level.OFF	= new Level(Level.OFF_INT, Level.OFF_VALUE);
var LogEvent = function(){this.init.apply(this, arguments);};
exports.LogEvent = LogEvent;

LogEvent.prototype.init = function(_logger, _level, _message, _exception) {

	this.logger = _logger;
	this.level = _level;
	this.message = _message;
	this.exception = _exception;

	this.timeStamp = new Date();
	this.timeStampInSeconds = Math.floor(this.timeStamp.getTime() / 1000);
};

LogEvent.prototype.getLogger = function() {

	return this.logger;
};

LogEvent.prototype.getTimeStamp = function() {

	return this.timeStamp;
};

LogEvent.prototype.getTimeStampInSeconds = function() {

	return this.timeStampInSeconds;
};

LogEvent.prototype.getLevel = function() {

	return this.level;
};

LogEvent.prototype.getMessage = function() {

	if(!this.message){ return ""; }
	return this.message;
};

LogEvent.prototype.getException = function() {

	return this.exception;
};
var StringBuilder = require("bn-lang-util").StringBuilder;
var SimpleDateFormat = require("bn-lang-util").SimpleDateFormat;

var PatternLayout = function(){this.init.apply(this, arguments);};
exports.PatternLayout = PatternLayout;

PatternLayout.prototype = new DefaultLayout();
PatternLayout.prototype.constructor = PatternLayout;
PatternLayout.parentclass = Array.prototype;

PatternLayout.prototype.init = function() {

	this.pattern = PatternLayout.DEFAULT_CONVERSION_PATTERN;
};

PatternLayout.prototype.format = function(_logEvent) {

	var _msg = new StringBuilder();

	var _regexp = new RegExp(PatternLayout.__PATTERN_REG_EXP__, "g");
	var _result;
	while((_result = _regexp.exec(this.pattern)) != null) {

		var _match = _result[0];
		var _padding = _result[1];
		var _truncation = _result[2];
		var _conversionCharacter = _result[3];
		//_result[4] // specifier with "{" and "}"
		var _specifier = _result[5];
		var _text = _result[6];

		var _replacement = "";
		if(_text) {

			_msg.append(_text);
			continue;
		}

		switch(_conversionCharacter) {

				case "c": // Logger name
					var _loggerName = _logEvent.getLogger().getName();
					if(_specifier) {

						var _precision = Number.parseInt(_specifier);
						var _loggerNameParts = _logEvent.getLogger().getName().split(".");
						if(_precision >= _loggerNameParts.length) {

							_replacement = _loggerName;

						} else {

							_replacement = _loggerNameParts.slice(_loggerNameParts.length - _precision).join(".");
						}

					} else {

						_replacement = _loggerName;
					}
					break;
				case "d": // Date
					var _dateFormat = PatternLayout.ISO8601_DATEFORMAT;
					if(_specifier) {

						_dateFormat = _specifier;
						// Pick up special cases
						if(_dateFormat == "ISO8601") {

							_dateFormat = PatternLayout.ISO8601_DATEFORMAT;

						} else if(_dateFormat == "ABSOLUTE") {

							_dateFormat = PatternLayout.ABSOLUTETIME_DATEFORMAT;

						} else if(_dateFormat == "DATE") {

							_dateFormat = PatternLayout.DATETIME_DATEFORMAT;
						}
					}
					_replacement = new SimpleDateFormat(_dateFormat).format(_logEvent.getTimeStamp());
					break;
				case "m": // Message
					_replacement = _logEvent.getMessage();
					break;
				case "n": // New line
					_replacement = "\n";
					break;
				case "p": // Level
					_replacement = _logEvent.getLevel().getName();
					break;
				case "r": // Milliseconds since startup
					_replacement = "" + _logEvent.getTimeStamp().getDifference(LogManager.START_TIMESTAMP);
					break;
				case "%": // Literal % sign
					_replacement = "%";
					break;
				default:
					_replacement = _match;
					break;
		}

		var len = 0;
		// truncation
		if(_truncation) {

			len = Number.parseInt(_truncation.substr(1));
			var strLen = _replacement.length;
			if(len < strLen) {

				_replacement = _replacement.substring(strLen - len, strLen);
			}
		}

		// padding
		if(_padding) {

			if(_padding.charAt(0) == "-") {

				len = Number.parseInt(_padding.substr(1));
				// Right pad with spaces
				while (_replacement.length < len) {

					_replacement += " ";
				}
			} else {

				len = Number.parseInt(_padding);
				// Left pad with spaces
				while (_replacement.length < len) {

					_replacement = " " + _replacement;
				}
			}
		}

		_msg.append(_replacement);
	}

	return _msg.toString();
};

PatternLayout.prototype.setConversionPattern = function(_pattern) {

	this.pattern = _pattern;
};

PatternLayout.TTCC_CONVERSION_PATTERN = "%r %p %c - %m%n";
PatternLayout.DEFAULT_CONVERSION_PATTERN = "%m%n";

PatternLayout.ISO8601_DATEFORMAT = "yyyy-MM-dd HH:mm:ss,SSS";
PatternLayout.DATETIME_DATEFORMAT = "dd MMM yyyy HH:mm:ss,SSS";
PatternLayout.ABSOLUTETIME_DATEFORMAT = "HH:mm:ss,SSS";

PatternLayout.__PATTERN_REG_EXP__ = "%(-?[0-9]+)?(\.?[0-9]+)?([cdmnpr%])(\{([^\}]+)\})?|([^%]+)";
var Hash = require("bn-lang").Hash;

var LogManager = function(){this.init.apply(this, arguments);};
exports.LogManager = LogManager;

LogManager.prototype.init = function() {};

LogManager.START_TIMESTAMP = new Date();
LogManager.LOGGERS = new Hash();

LogManager.getLogger = function(_loggerName) {

// TODO - hierarchy
	var contains = LogManager.LOGGERS.containsKey(_loggerName);
	if (!contains) {

		LogManager.LOGGERS.put(_loggerName, new Logger(_loggerName));
	}

	return LogManager.LOGGERS.get(_loggerName);
};

LogManager.getObjectLogger = function(_loggerName) {

// TODO - hierarchy
	var contains = LogManager.LOGGERS.containsKey(_loggerName);
	if (!contains) {

		LogManager.LOGGERS.put(_loggerName, new ObjectLogger(_loggerName));
	}

	return LogManager.LOGGERS.get(_loggerName);
};
var Logger = function(){this.init.apply(this, arguments);};
exports.Logger = Logger;

Logger.prototype.init = function(_name) {

	this.name = _name;
	this.appenders = [];
	this.effectiveLevel = Level.FATAL;
};

Logger.prototype.log = function(_level, _message, _exception) {

	if(_level.isGreaterOrEqual(this.effectiveLevel)) {

		var _logEvent = new LogEvent(this, _level, _message, _exception);
		this.appenders.each(function(_appender) {

			_appender.doAppend(_logEvent);
		});
	}
};

Logger.prototype.getName = function() {

	return this.name;
};

Logger.prototype.addAppender = function(_appender) {

	this.appenders.add(_appender);
};

Logger.prototype.removeAppender = function(_appender) {

	this.appenders.remove(_appender);
};

Logger.prototype.removeAllAppenders = function() {

	this.appenders.clear();
};

Logger.prototype.getEffectiveLevel = function() {

	return this.effectiveLevel;
};

Logger.prototype.setEffectiveLevel = function(_level) {

	this.effectiveLevel = _level;
	return this;
};

Logger.prototype.trace = function(_message, _exception) {

	this.log(Level.TRACE, _message, _exception);
};

Logger.prototype.debug = function(_message, _exception) {

	this.log(Level.DEBUG, _message, _exception);
};

Logger.prototype.info = function(_message, _exception) {

	this.log(Level.INFO, _message, _exception);
};

Logger.prototype.warn = function(_message, _exception) {

	this.log(Level.WARN, _message, _exception);
};

Logger.prototype.error = function(_message, _exception) {

	this.log(Level.ERROR, _message, _exception);
};

Logger.prototype.fatal = function(_message, _exception) {

	this.log(Level.FATAL, _message, _exception);
};

Logger.prototype.isTraceEnabled = function() {

	return Level.TRACE.isGreaterOrEqual(this.getLevel());
};

Logger.prototype.isDebugEnabled = function() {

	return Level.DEBUG.isGreaterOrEqual(this.getLevel());
};

Logger.prototype.isInfoEnabled = function() {

	return Level.INFO.isGreaterOrEqual(this.getLevel());
};

Logger.prototype.isWarnEnabled = function() {

	return Level.WARN.isGreaterOrEqual(this.getLevel());
};

Logger.prototype.isErrorEnabled = function() {

	return Level.ERROR.isGreaterOrEqual(this.getLevel());
};

Logger.prototype.isFatalEnabled = function() {

	return Level.FATAL.isGreaterOrEqual(this.getLevel());
};
var Hash = require("bn-lang").Hash;
var StringBuilder = require("bn-lang-util").StringBuilder;

var ObjectLogger = function(){this.init.apply(this, arguments);};
exports.ObjectLogger = ObjectLogger;

ObjectLogger.prototype = new Logger();
ObjectLogger.prototype.constructor = ObjectLogger;
ObjectLogger.parentclass = Logger.prototype;

ObjectLogger.prototype.init = function(_name) {

	ObjectLogger.parentclass.init.apply(this, [_name]);
};

ObjectLogger.prototype.addObject = function(_name, _instance, _options) {

	var options = {};
	if(!_options) {

		options.beforeAndAfter = true;
		options.before = false;
		options.after = false;

		options.all = true;

	} else {

		options.beforeAndAfter = _options.beforeAndAfter || false;
		options.before = _options.before || false;
		options.after = _options.after || false;

		options.all = _options.all || true;
		options.funcs = _options.funcs || [];

		if(_options.funcs && _options.funcs.length) {

			options.all = false;
		}
	}

	var funcs = new Hash();

	var me = this;
	for(var prop in _instance) {

		if(typeof _instance[prop] == "function") {

			var method = _instance[prop];

			funcs.put(prop, _instance[prop]);
		}
	}

	funcs.each(function(_value, _key) {

		_instance[_key+"_tmp"] = _value;

		var _before = options.before;
		var _after = options.after;
		var _beforeAndAfter = options.beforeAndAfter;

		if(!options.all) {

			if(!options.funcs.any(function(_funcObj) {

				return _funcObj.name == _key;
				
			})) {

				funcs.$continue();
			}

			var logFunc = options.funcs.detect(function(_funcObj) {
			
				return _funcObj.name == _key;
			});

			_before = logFunc.before;
			_after = logFunc.after;
			_beforeAndAfter = logFunc.beforeAndAfter;
		}

		// before
		if(_before) {

			_instance[_key] = function() {

				var sb = new StringBuilder();

				sb.append("\n\t");
				sb.append("[BEFORE] " + _name + ": " + _key);
				sb.append("\n");
				sb.append(me.getArgs(arguments));

				me.debug(sb.toString());

				return _instance[_key+"_tmp"].apply(this, arguments);
			};
		}

		// after
		if(_after) {

			_instance[_key] = function() {

				var sb = new StringBuilder();

				var _return = _instance[_key+"_tmp"].apply(this, arguments);

				sb.append("\n\t[AFTER ] " + _name + ": " + _key);
				sb.append("\n\t\t" + _return);

				me.debug(sb.toString());

				return _return;
			};
		}

		// before and after
		if(_beforeAndAfter) {

			_instance[_key] = function() {

				var sb = new StringBuilder();

				sb.append("\n\t");
				sb.append("[BEFORE] " + _name + ": " + _key);
				sb.append("\n");
				sb.append(me.getArgs(arguments));

				var _return = _instance[_key+"_tmp"].apply(this, arguments);

				sb.append("\n\t[AFTER ] " + _name + ": " + _key);
				sb.append("\n\t\t" + _return);

				me.debug(sb.toString());

				return _return;
			};
		}
	});
};

ObjectLogger.prototype.getArgs = function(_args) {

	var sb = new StringBuilder();

	sb.append("\t\tArguments:\n");
	Enumerable.convertToArray(_args).each(function(_value) {

		sb.append("\t\t\t" + _value);
	});

	return sb.toString();
};
var Hash = require("bn-lang").Hash;

var LogConfiguration = function(){this.init.apply(this, arguments);};
exports.LogConfiguration = LogConfiguration;

LogConfiguration.prototype.init = function(_configuration) {

	var configuration = _configuration;
	this.getConfiguration = function() {

		return configuration;
	};
};

LogConfiguration.prototype.configure = function() {

	var _appenersMap = new Hash();

	var i=0;

	var _appenders = this.getConfiguration().appenders;
	for(i=0;i<_appenders.length;i++) {

		_appender = _appenders[i];

		var realAppender = new _appender.type(_appender.name);

		if(_appender.threshold) {

			realAppender.setThreshold(Level.toLevel(_appender.threshold));
		}

		if(_appender.layout) {

			var realLayout = new _appender.layout();

			if(_appender.layout_conversion) {

				realLayout.setConversionPattern(_appender.layout_conversion);
			}

			realAppender.setLayout(realLayout);
		}
		_appenersMap.put(realAppender.getName(), realAppender);
	}

	var _loggers = this.getConfiguration().loggers;
	for(i=0;i<_loggers.length;i++) {

		var logger = _loggers[i];

		var realLogger = LogManager.getLogger(logger.name);
		if(logger.effectiveLevel) {
		
			realLogger.setEffectiveLevel(Level.toLevel(logger.effectiveLevel));
		}

		if(logger.appenders) {

			for(var j=0;j<logger.appenders.length;j++) {
			
				realLogger.addAppender(_appenersMap.get(logger.appenders[j]));
			}
		}
	}
};
