var Hash = require("bn-lang").Hash;
var StringBuilder = require("bn-lang-util").StringBuilder;

var ObjectLogger = function(){this.init.apply(this, arguments);};
exports.ObjectLogger = ObjectLogger;

ObjectLogger.prototype = new Logger();
ObjectLogger.prototype.constructor = ObjectLogger;
ObjectLogger.parentclass = Logger.prototype;

ObjectLogger.prototype.init = function(_name) {

	ObjectLogger.parentclass.init.apply(this, [_name]);
};

ObjectLogger.prototype.addObject = function(_name, _instance, _options) {

	var options = {};
	if(!_options) {

		options.beforeAndAfter = true;
		options.before = false;
		options.after = false;

		options.all = true;

	} else {

		options.beforeAndAfter = _options.beforeAndAfter || false;
		options.before = _options.before || false;
		options.after = _options.after || false;

		options.all = _options.all || true;
		options.funcs = _options.funcs || [];

		if(_options.funcs && _options.funcs.length) {

			options.all = false;
		}
	}

	var funcs = new Hash();

	var me = this;
	for(var prop in _instance) {

		if(typeof _instance[prop] == "function") {

			var method = _instance[prop];

			funcs.put(prop, _instance[prop]);
		}
	}

	funcs.each(function(_value, _key) {

		_instance[_key+"_tmp"] = _value;

		var _before = options.before;
		var _after = options.after;
		var _beforeAndAfter = options.beforeAndAfter;

		if(!options.all) {

			if(!options.funcs.any(function(_funcObj) {

				return _funcObj.name == _key;
				
			})) {

				funcs.$continue();
			}

			var logFunc = options.funcs.detect(function(_funcObj) {
			
				return _funcObj.name == _key;
			});

			_before = logFunc.before;
			_after = logFunc.after;
			_beforeAndAfter = logFunc.beforeAndAfter;
		}

		// before
		if(_before) {

			_instance[_key] = function() {

				var sb = new StringBuilder();

				sb.append("\n\t");
				sb.append("[BEFORE] " + _name + ": " + _key);
				sb.append("\n");
				sb.append(me.getArgs(arguments));

				me.debug(sb.toString());

				return _instance[_key+"_tmp"].apply(this, arguments);
			};
		}

		// after
		if(_after) {

			_instance[_key] = function() {

				var sb = new StringBuilder();

				var _return = _instance[_key+"_tmp"].apply(this, arguments);

				sb.append("\n\t[AFTER ] " + _name + ": " + _key);
				sb.append("\n\t\t" + _return);

				me.debug(sb.toString());

				return _return;
			};
		}

		// before and after
		if(_beforeAndAfter) {

			_instance[_key] = function() {

				var sb = new StringBuilder();

				sb.append("\n\t");
				sb.append("[BEFORE] " + _name + ": " + _key);
				sb.append("\n");
				sb.append(me.getArgs(arguments));

				var _return = _instance[_key+"_tmp"].apply(this, arguments);

				sb.append("\n\t[AFTER ] " + _name + ": " + _key);
				sb.append("\n\t\t" + _return);

				me.debug(sb.toString());

				return _return;
			};
		}
	});
};

ObjectLogger.prototype.getArgs = function(_args) {

	var sb = new StringBuilder();

	sb.append("\t\tArguments:\n");
	Enumerable.convertToArray(_args).each(function(_value) {

		sb.append("\t\t\t" + _value);
	});

	return sb.toString();
};
